﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectSeaplane.DrawningObjects;
using ProjectSeaplane.MovementStrategy;
using ProjectSeaplane.Generics;
using System.IO;

namespace ProjectSeaplane.Generics
{
    internal class PlanesGenericStorage
    {
        /// <summary>
        /// Разделитель для записи ключа и значения элемента словаря
        /// </summary>
        private static readonly char _separatorForKeyValue = '|';
        /// <summary>
        /// Разделитель для записей коллекции данных в файл
        /// </summary>
        private readonly char _separatorRecords = ';';
        /// <summary>
        /// Разделитель для записи информации по объекту в файл
        /// </summary>
        private static readonly char _separatorForObject = ':';

        /// <summary>
        /// Сохранение информации по автомобилям в хранилище в файл
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - сохранение прошло успешно, false - ошибка при сохранении данных</returns>
        public bool SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            StringBuilder data = new();
            foreach (KeyValuePair<string,
            PlanesGenericCollection<DrawningPlane, DrawningObjectPlane>> record in _planeStorages)
            {
                StringBuilder records = new();
                foreach (DrawningPlane? elem in record.Value.GetPlanes)
                {
                    records.Append($"{elem?.GetDataForSave(_separatorForObject)}{_separatorRecords}");
                }
                data.AppendLine($"{record.Key}{_separatorForKeyValue}{records}");
            }
            if (data.Length == 0)
            {
                return false;
            }
            using (StreamWriter writer = new StreamWriter(filename, false))
            {
                writer.WriteLine("PlaneStorage");
                writer.Write(data.ToString());
            }
            
            return true;
        }

        /// <summary>
        /// Загрузка информации по автомобилям в хранилище из файла
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - загрузка прошла успешно, false - ошибка при загрузке данных</returns>
        public bool LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                return false;
            }

            using (StreamReader reader = new StreamReader(filename))
            {
                string cheker = reader.ReadLine();
                if (cheker == null)
                {
                    return false;
                }
                if (!cheker.StartsWith("PlaneStorage"))
                {
                    return false;
                }
                _planeStorages.Clear();
                string strs;
                bool firstinit = true;
                while ((strs = reader.ReadLine()) != null) 
                {
                    if (strs == null && firstinit)
                    {
                        return false;
                    }
                    if (strs == null)
                    {
                        return false;
                    }
                    firstinit = false;
                    string name = strs.Split(_separatorForKeyValue)[0];
                    PlanesGenericCollection<DrawningPlane, DrawningObjectPlane> collection = new(_pictureWidth, _pictureHeight);
                    foreach (string data in strs.Split(_separatorForKeyValue)[1].Split(_separatorRecords))
                    {
                        DrawningPlane? plane =
                        data?.CreateDrawningPlane(_separatorForObject, _pictureWidth, _pictureHeight);
                        if (plane != null)
                        {
                            if (!(collection + plane))
                            {
                                return false;
                            }
                        }
                    }
                    _planeStorages.Add(name, collection);
                }
                return true;
            }
        }

        readonly Dictionary<string, PlanesGenericCollection<DrawningPlane, DrawningObjectPlane>> _planeStorages;
        public List<string> Keys => _planeStorages.Keys.ToList();
        private readonly int _pictureWidth;
        private readonly int _pictureHeight;
        public PlanesGenericStorage(int pictureWidth, int pictureHeight)
        {
            _planeStorages = new Dictionary<string, PlanesGenericCollection<DrawningPlane, DrawningObjectPlane>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
        public void AddSet(string name)
        {
            if (_planeStorages.ContainsKey(name)) return;
            _planeStorages[name] = new PlanesGenericCollection<DrawningPlane, DrawningObjectPlane>(_pictureWidth, _pictureHeight);
        }

        public void DelSet(string name)
        {
            if (!_planeStorages.ContainsKey(name)) return;
            _planeStorages.Remove(name);
        }

        public PlanesGenericCollection<DrawningPlane, DrawningObjectPlane>?
        this[string ind]
        {
            get
            {
                if (_planeStorages.ContainsKey(ind)) return _planeStorages[ind];
                return null;
            }
        }

    }

}
